from typing import AsyncGenerator

from mach_client import (
    AccountManager,
    AssetServer,
    MachClient,
    RiskManager,
    SupportedChain,
)

from ..log import Logger
from .aave import Aave, AaveRunner
from .event import AaveEvent
from .rebalance_manager import RebalanceManager


async def run(
    *,
    client: MachClient,
    asset_server: AssetServer,
    accounts: AccountManager,
    rebalance_manager: RebalanceManager,
    # When rebalancing, if swapping to higher rate tokens fails, should we try swapping to a lower rate token or keep what we have?
    filter_lower_rate_tokens: bool,
    risk_manager: RiskManager,
    logger: Logger,
) -> AsyncGenerator[AaveEvent, None]:
    chains = frozenset(
        (
            SupportedChain.ARBITRUM.value,
            SupportedChain.BASE.value,
            SupportedChain.OPTIMISM.value,
        )
    )

    aave = await Aave.create(
        client=client, asset_server=asset_server, chains=chains, logger=logger
    )

    runner = AaveRunner(
        aave=aave,
        accounts=accounts,
        rebalance_manager=rebalance_manager,
        filter_lower_rate_tokens=filter_lower_rate_tokens,
        risk_manager=risk_manager,
        logger=logger,
    )

    async with runner as events:
        async for event in events:
            yield event
